@@ -100,6 +100,8 @@ static void regionExpose(Widget, PaintWidget, XEvent *, Boolean *);
 		mm(pw->paint.region.rotMat, m, pw->paint.region.mat);	\
 	} while (0)
 
+static int transparent = 0;
+
 static pwMatrix matIdentity =
 {
     {1, 0},
@@ -126,16 +128,14 @@ doCallbacks(PaintWidget pw, int flag)
 static void 
 mm(pwMatrix a, pwMatrix b, pwMatrix r)
 {
-    float t00, t10, t01, t11;
-
+    float t00, t01, t10, t11;
     t00 = a[0][0] * b[0][0] + a[0][1] * b[1][0];
     t01 = a[0][0] * b[0][1] + a[0][1] * b[1][1];
     t10 = a[1][0] * b[0][0] + a[1][1] * b[1][0];
     t11 = a[1][0] * b[0][1] + a[1][1] * b[1][1];
-
     r[0][0] = t00;
-    r[1][0] = t10;
     r[0][1] = t01;
+    r[1][0] = t10;
     r[1][1] = t11;
 }
 
@@ -166,6 +166,83 @@ buildSources(PaintWidget pw)
     }
 }
 
+void 
+RegionTransparency(PaintWidget pw, int mode)
+{
+    Widget w;
+    XImage *src1, *src2;
+    Pixel pix1, pix2;
+    XRectangle rect;
+    int rwidth, rheight, pwidth, pheight;
+    int px, py, rx, ry, x, y, i, j, bytes_per_pixel;
+
+    w = pw->paint.region.child;
+    if (!pw->paint.region.source || !w) {
+       transparent = 0;
+       return;
+    }
+
+    transparent = !mode;
+    px = pw->paint.region.rect.x;
+    py = pw->paint.region.rect.y;
+    if (px<0) {
+       rx = -px;
+       px = 0;
+       rwidth = w->core.width - rx;
+    } else {
+       rx = 0;
+       rwidth = w->core.width;
+    }
+    if (py<0) {
+       ry = -py;
+       py = 0;
+       rheight = w->core.height - ry;
+    } else {
+       ry = 0;
+       rheight = w->core.height;
+    }
+    if (px+rwidth>pw->paint.drawWidth) 
+       rwidth = pw->paint.drawWidth - px;
+    if (py+rheight>pw->paint.drawHeight) 
+       rheight = pw->paint.drawHeight - py;
+    if (rwidth<=0 || rheight<=0)
+       return;
+
+    src1 = XGetImage(XtDisplay(pw), pw->paint.region.source,
+       	             rx, ry, rwidth, rheight,
+		     AllPlanes, ZPixmap);
+    if (mode) {
+       XPutImage(XtDisplay(pw), XtWindow(w), 
+          pw->paint.region.fg_gc == None ?
+	     pw->paint.tgc : pw->paint.region.fg_gc,
+          src1, 0, 0, rx, ry, rwidth, rheight);
+       XDestroyImage(src1);
+       return;
+    }
+
+    src2 = XGetImage(XtDisplay(pw), GET_PIXMAP(pw),
+		     px, py, rwidth, rheight,
+		     AllPlanes, ZPixmap);
+    bytes_per_pixel = src1->bits_per_pixel/8;
+
+    for (y=0; y<rheight; y++)
+       for (x=0; x<rwidth; x++) if ((x+y)&1) {
+	  i = bytes_per_pixel * x + src1->bytes_per_line * y;
+	  for (j=0; j<bytes_per_pixel; j++) {
+	     src1->data[i] = src2->data[i];
+	     ++i;
+       	  }
+       }
+
+    XPutImage(XtDisplay(pw), XtWindow(w), 
+       pw->paint.region.fg_gc == None ?
+	  pw->paint.tgc : pw->paint.region.fg_gc,
+       src1, 0, 0, rx, ry, rwidth, rheight);
+
+    XDestroyImage(src1);
+    XDestroyImage(src2);
+}
+
 static void 
 resizeImg(PaintWidget pw, pwMatrix inv, Pixmap * pix, XImage * pixSrc,
 	  Pixmap * msk, XImage * mskSrc, Pixmap * shape)
@@ -228,8 +305,8 @@ resizeImg(PaintWidget pw, pwMatrix inv, Pixmap * pix, XImage * pixSrc,
     for (y = 0, dy = fy; y < height; y++, dy++) {
 	for (x = 0, dx = fx; x < width; x++, dx++) {
 	    XFORM(dx, dy, inv, sx, sy);
-	    ix = (sx + cx);
-	    iy = (sy + cy);
+	    ix = (sx + cx + 0.5);
+	    iy = (sy + cy + 0.5);
 	    if (ix >= 0 && ix < sourceW && iy >= 0 && iy < sourceH) {
 		xxPutPixel(pixDst, x, y, xxGetPixel(pixSrc, ix, iy));
 		if (mskSrc != NULL)
@@ -816,6 +893,13 @@ regionGrab(Widget w, PaintWidget pw, XMotionEvent * event, Boolean * junk)
         motionExtern((Widget)pw, (XEvent *) event,
 	   pw->paint.region.rect.x, 
            pw->paint.region.rect.y + pw->paint.region.rect.height/zoom - 1, 0);
+        /* 0xF8 is Mod?Mask */
+	if ((zoom==1) && (event->state & 0xF8))
+           RegionTransparency(pw, 0);
+	else {
+	   if (zoom==1 && transparent)
+	      RegionTransparency(pw, 1);
+	}
     }
 }
 
@@ -1861,3 +1945,4 @@ RegionMove(PaintWidget pw, int dx, int dy)
 
     drawRegionBox(pw, False);
 }
+
