@@ -50,7 +50,7 @@
 #include "protocol.h"
 
 #define MAX_PATH	1024
-#define NUM_FORMATS VALUE-IMAGE_FORMATS-1
+#define NUM_FORMATS END_IMAGE_FORMATS-IMAGE_FORMATS
 
 #ifndef NOSTDHDRS
 #include <stdlib.h>
