@@ -113,13 +113,15 @@ Image *ImageNew(int width, int height);
 Image *ImageNewCmap(int width, int height, int size);
 Image *ImageNewBW(int width, int height);
 Image *ImageNewGrey(int width, int height);
+Image *ImageToRGB(Image * image);
 void ImageMakeMask(Image * image);
 void ImageDelete(Image * image);
-Image *ImageToRGB(Image * image);
+int ImageToMemory(Image * image);
 
 #ifdef _XtIntrinsic_h
 Image *PixmapToImage(Widget w, Pixmap pix, Colormap cmap);
 void PixmapToImageMask(Widget w, Image * image, Pixmap mask);
+Image *ClipboardGetImage(int num);
 Boolean ImageToPixmap(Image * image, Widget w, Pixmap * pix, Colormap * cmap);
 Pixmap MaskDataToPixmap(Widget w, int width, int height,
 			char *data, XRectangle * rect);
@@ -157,6 +159,7 @@ Image *ImageQuantize(Image * input);
 Image *ImageUserDefined(Image * input);
 Image *ImageTilt(Image * input);
 Image *ImageDirectionalFilter(Image * input);
+Image *ImageFromFile(char *file);
 
 /* texture.c */
 Image *draw_plasma(int w, int h);
