@@ -1,7 +1,7 @@
 /* +-------------------------------------------------------------------+ */
 /* | Copyright 1993, David Koblas (koblas@netcom.com)		       | */
 /* | Copyright 1995, 1996 Torsten Martinsen (bullestock@dk-online.dk)  | */
-/* |							       | */
+/* |								       | */
 /* | Permission to use, copy, modify, and to distribute this software  | */
 /* | and its documentation for any purpose is hereby granted without   | */
 /* | fee, provided that the above copyright notice appear in all       | */
@@ -45,6 +45,8 @@ typedef struct {
 
 #define BUFSIZE 1024
 
+static void *dl_filter = NULL;
+
 /*
 **  Some real image processing
  */
@@ -80,16 +82,22 @@ static PaintMenuItem predefMenu[] =
 {
 #define PREDEF_FILTERS 0
     MI_SIMPLE("filters"),
-#define PREDEF_LAYERS 1
+#define PREDEF_IMAGES 1
+    MI_SIMPLE("images"),
+#define PREDEF_3D_CURVES 2
+    MI_SIMPLE("3d_curves"),
+#define PREDEF_3D_SURFACES 3
+    MI_SIMPLE("3d_surfaces"),
+#define PREDEF_LAYERS 4
     MI_SIMPLE("layers"),
-#define PREDEF_SEPARATOR 2
+#define PREDEF_PROCEDURES 5
+    MI_SIMPLE("procedures"),
+#define PREDEF_SEPARATOR 6
     MI_SEPARATOR(),
-#define PREDEF_HELP 3
+#define PREDEF_HELP 7
     MI_SIMPLECB("help", HelpDialog, "script_editor"),
 };
 
-static void *dl_handle[PREDEF_SEPARATOR] = { NULL, NULL };
-
 static PaintMenuBar printMenuBar[] =
 {
     {None, "script_files", XtNumber(fileMenu), fileMenu},
@@ -1655,9 +1663,19 @@ compileCallback(Widget w, XtPointer wlArg, XtPointer junk)
 {
    LocalInfo *info = (LocalInfo *) wlArg;
    FILE *fd;
+   void (* proc)(Widget wid);
+   Image * (* iproc)();
+   Image * image;
+   Screen *screen;
+   Visual *visual;
+   Pixmap pix;
+   Colormap cmap;
    char *tmp, *ptr, *text;
    char cmd[512];
+   char header[30];
    char * error;
+   void *dl_handle = NULL;
+   int i;
 
    /* create temporary name */
    fd= openTemp(&tmp);
@@ -1671,22 +1689,40 @@ compileCallback(Widget w, XtPointer wlArg, XtPointer junk)
    sprintf(cmd, "%s.c", tmp);
    fd = fopen(cmd, "w");
    if (!fd) return;
+
    ptr = text;
+   i = 0;
    while (*ptr) {
       fputc(*ptr, fd);
+      if (i <= 25) {
+         header[i] = *ptr;
+         if (*ptr == '\n') header[i] = '\0';
+	 i++;
+      }
       ++ptr;
    }
    fclose(fd);
+   header[i] = '\0';
+
+   info->mode = PREDEF_IMAGES;
+   if (strncasecmp(header, "/* Xpaint-image */", 18) == 0)
+      info->mode = PREDEF_IMAGES;
+   if (strncasecmp(header, "/* Xpaint-filter */", 19) == 0)
+      info->mode = PREDEF_FILTERS;
+   if (strncasecmp(header, "/* Xpaint-procedure */", 22) == 0)
+      info->mode = PREDEF_PROCEDURES;
+
 
    /* free any previous dll */
-   if (dl_handle[info->mode]) {
-      dlclose(dl_handle[info->mode]);
-      dl_handle[info->mode] = NULL;
+   if (info->mode == PREDEF_FILTERS && dl_filter) {
+      dlclose(dl_filter);
+      dl_filter = NULL;
    }
 
    /* compile C script */
-   sprintf(cmd, "cd /tmp\ngcc -c %s.c -o %s.o\n"
+   sprintf(cmd, "cd /tmp\ngcc -I%s -c %s.c -o %s.o\n"
                 "gcc -shared -Wl,-soname,%s.so %s.o -o %s.so\n",
+	        SHAREDIR"/include",
                 tmp, tmp, strstr(tmp, "XPaint"), tmp, tmp);
    XtVaSetValues(info->cclog, XtNstring, cmd, NULL);
    pipe_command(cmd, NULL, CollectOutput, CollectOutput);
@@ -1699,19 +1735,58 @@ compileCallback(Widget w, XtPointer wlArg, XtPointer junk)
 
    /* load dll library */
    sprintf(cmd, "%s.so", tmp);
-   dl_handle[info->mode] = dlopen(cmd, RTLD_LAZY);
+   if (info->mode == PREDEF_FILTERS)
+      dl_handle = dl_filter = dlopen(cmd, RTLD_LAZY);
+   else
+      dl_handle = dlopen(cmd, RTLD_LAZY);
 
    /* clean dll .so file */
    unlink(cmd);
    removeTemp();
 
-   if (dl_handle[info->mode])
+   if (dl_handle)
       sprintf(cmd, "echo \"%s\"",
 	      msgText[C_SCRIPT_SUCCESSFULLY_COMPILED_AND_LINKED]);
    else
       sprintf(cmd, "echo \"\"; echo \"%s\"; echo \"%s\"",
               msgText[C_SCRIPT_COULD_NOT_BE_COMPILED_OR_LINKED], dlerror());
    pipe_command(cmd, NULL, CollectOutput, CollectOutput);
+
+   if (info->mode == PREDEF_IMAGES) {
+      if (!dl_handle) return;
+      iproc = dlsym(dl_handle, "ImageCreate");
+      if ((error = dlerror()) != NULL || !iproc) {
+	 if (!error) error = msgText[UNABLE_TO_CREATE_IMAGE];
+         Notice(Global.toplevel, error);
+         return;
+      }
+      pix = None;
+      image = iproc();
+      dlclose(dl_handle);
+      dl_handle = NULL;
+      if (image) {
+	 screen = XtScreen(Global.toplevel);
+	 visual = DefaultVisualOfScreen(screen);
+	 cmap = XCreateColormap(XtDisplay(Global.toplevel), 
+		    RootWindowOfScreen(screen), visual, AllocNone);
+	 ImageToPixmap(image, Global.toplevel, &pix, &cmap);
+      }
+      if (!pix) {
+         Notice(Global.toplevel, msgText[UNABLE_TO_CREATE_IMAGE]);
+	 return;
+      }
+      graphicCreate(makeGraphicShell(Global.toplevel), 0, 0, -1, pix, cmap);
+      return;
+   }
+
+   if (info->mode == PREDEF_PROCEDURES) {
+      if (!dl_handle) return;
+      proc = dlsym(dl_handle, "PaintProcedure");
+      if (proc) proc(Global.toplevel);
+      dlclose(dl_handle);
+      dl_handle = NULL;
+      return;
+   }
 }
 
 static void
@@ -1868,13 +1943,24 @@ saveasFileCallback(Widget w, XtPointer wlArg, XtPointer junk)
 }
 
 static void
-loadFilterCallback(Widget w, XtPointer wlArg, XtPointer junk)
+loadScriptCallback(Widget w, XtPointer wlArg, XtPointer junk)
 {
    LocalInfo *info = (LocalInfo *) wlArg;
-   info->mode = 0;
+   char directory[256];
+   int i;
+
+   info->mode = PREDEF_IMAGES;
+   for (i=0; i<PREDEF_SEPARATOR; i++)
+      if (w == predefMenu[i].widget) {
+	 info->mode = i;
+	 break;
+      }
+
    Global.explore = True;
-   GetFileName(GetShell(w), 3, 
-            SHAREDIR"/filters/", (XtCallbackProc) loadFileCallbackOK, wlArg);
+   sprintf(directory, "%s/%s/", SHAREDIR"/c_scripts", 
+	              predefMenu[info->mode].name);
+   GetFileName(GetShell(w), 3, directory, 
+      (XtCallbackProc) loadFileCallbackOK, wlArg);
    Global.explore = False;
 }
 
@@ -1887,6 +1973,7 @@ ScriptEditor(Widget w, Widget paint)
     Widget topform, midform, botform;
     Widget bar, cancel, compile;
     LocalInfo *info = XtNew(LocalInfo);
+    int i;
 
     head = info;
     info->mode = 0;
@@ -1895,7 +1982,7 @@ ScriptEditor(Widget w, Widget paint)
     dpy = XtDisplay(GetShell(w));
     root = RootWindowOfScreen(XtScreen(GetShell(w)));
 
-    info->shell = XtVisCreatePopupShell("XPaint / C script", 
+    info->shell = XtVisCreatePopupShell("XPaint / C scripts", 
                                   topLevelShellWidgetClass,
 				  Global.toplevel, NULL, 0);
     pane = XtVaCreateManagedWidget("pane", panedWidgetClass, info->shell,
@@ -1968,8 +2055,9 @@ ScriptEditor(Widget w, Widget paint)
     XtAddCallback(cancel, XtNcallback,
                   (XtCallbackProc)closeCallback, (XtPointer) info);
 
-    XtAddCallback(predefMenu[PREDEF_FILTERS].widget, XtNcallback, 
-		     (XtCallbackProc) loadFilterCallback, (XtPointer) info);
+    for (i=0; i<PREDEF_SEPARATOR; i++)
+       XtAddCallback(predefMenu[i].widget, XtNcallback, 
+		     (XtCallbackProc) loadScriptCallback, (XtPointer) info);
 
     XtRealizeWidget(info->shell);
     XtPopup(info->shell, XtGrabNone);
@@ -2017,7 +2105,7 @@ CanvasImage()
 Boolean
 isFilterDefined()
 {
-    return (dl_handle[0])? True : False;
+    return (dl_filter)? True : False;
 }
 
 Image *
@@ -2029,7 +2117,9 @@ ImageUserDefined(Image * input)
     int x, y;
     void (* proc)(Image *, Image *);
     
-    proc = dlsym(dl_handle[0], "FilterProcess");
+    if (!dl_filter) return NULL;
+
+    proc = dlsym(dl_filter, "FilterProcess");
     if ((error = dlerror()) != NULL) {
        Notice(Global.toplevel, error);
        return input;
