@@ -139,6 +139,19 @@ static XrmOptionDescRec options[] =
 **  Public query functions for application defaults
  */
 
+void 
+SetDefaultWHZ(int w, int h, int zoom)
+{
+    char str[40];
+    if (w<=0) w = 1;
+    if (h<=0) h = 1;
+    if (zoom<=0) zoom = -1;
+    if (zoom>32) zoom = 32;
+    sprintf(str, "%dx%d", w, h);
+    appInfo.size = strdup(str);
+    appInfo.zoom = zoom;
+}
+
 void 
 GetDefaultWH(int *w, int *h)
 {
@@ -470,6 +483,39 @@ String GetAppDefaultFile()
         return strdup("XPaint");
 }
 
+
+static void
+checkRCsize()
+{
+    FILE *fd;
+    char buf[256];
+    char *ptr;
+    int w, h, i;
+    if (!strcmp(appInfo.size, "640x480")) {
+        if (appInfo.rcFile)
+	    strcpy(buf, appInfo.rcFile);
+	else
+	    strcpy(buf, ".XPaintrc");
+	if ((fd = fopen(buf, "r"))) {
+	    i = 0;
+            while (!feof(fd)) {
+                fgets(buf, 250, fd);
+		if (*buf && isspace(*buf)) ++i;
+		if (!strncasecmp(buf, "defaultsize", 11)) {
+		    ptr = buf + 12;
+		    while (isspace(*ptr)) ++ptr;
+	            if (sscanf(ptr, "%dx%d", &w, &h) == 2) {
+	                 SetDefaultWHZ(w, h, appInfo.zoom);
+		    }
+		    break;
+		}
+		if (i >= 4) break;
+	    }
+	    fclose(fd);
+	}
+    }
+}
+
 int
 main(int argc, char *argv[])
 {
@@ -640,7 +686,7 @@ main(int argc, char *argv[])
 
     /* destroy our temporary widget */
     XtDestroyWidget(temp);
-	
+
     /*
     **  Create the application context
      */
@@ -651,10 +697,12 @@ main(int argc, char *argv[])
 
     XtGetApplicationResources(toplevel, (XtPointer) & appInfo,
 			      resources, XtNumber(resources), NULL, 0);
-	 
+
     if (argc != 1 && argv[1][0] == '-')
 	usage(argv[0], "Invalid option");
-	 
+
+    checkRCsize();
+
     /*
     **  A little initialization
      */
@@ -675,7 +723,8 @@ main(int argc, char *argv[])
     Global.dashlist[0] = '\0';
     Global.dashnumber = 0;
     Global.dashoffset = 0;
-
+    Global.numregions = 0;
+    Global.regiondata = NULL;
 
     /*
     **
@@ -764,19 +813,6 @@ main(int argc, char *argv[])
      */
     XtRealizeWidget(toplevel);
 
-#ifdef XAW95
-/* This is a hack to avoid ugly scrollbars with Xaw95 widgets */
-    {
-    Window root;
-    Widget w = toplevel;
-    int x, y;
-    unsigned int width, height, bw, depth;
-    XGetGeometry(XtDisplay(w), XtWindow(w),
-		 &root, &x, &y, &width, &height, &bw, &depth);
-    XResizeWindow(XtDisplay(w), XtWindow(w), width+3, height+3);
-    }
-#endif
-
     /*
     **  Now open any file on the command line
      */
