@@ -107,6 +107,7 @@ popupMainAction(Widget w, XEvent * event, String * params, Cardinal * nparams)
     Widget editmenu;
     Position x, y;
     Dimension width;
+    int zoom;
 
     if (strcmp(XtName(w), "paint") || ThereIsNoMenuBar() || IsFullPopup()) {
       mainpopup:
@@ -117,8 +118,9 @@ popupMainAction(Widget w, XEvent * event, String * params, Cardinal * nparams)
     XtVaGetValues(w, XtNmenuwidgets, &wlist, NULL);
     if (!wlist) goto mainpopup;
     if (!wlist[8]) goto mainpopup;
+    XtVaGetValues(w, XtNzoom, &zoom, NULL);
+    XtTranslateCoords(w, zoom*event->xbutton.x, zoom*event->xbutton.y, &x, &y);
     editmenu = GetShell(wlist[8]);
-    XtTranslateCoords(w, event->xbutton.x, event->xbutton.y, &x, &y);
     XtVaGetValues(editmenu, XtNwidth, &width, NULL);
     XtMoveWidget(editmenu, x-width/2, y-8);
     XtPopup(editmenu, XtGrabNone);
