@@ -393,7 +393,7 @@ saveConfigOkCallback(Widget il, LocalInfo * info, char *file)
 {
     WidgetList children;
     int nchildren;
-    int i;
+    int i, w, h;
     FILE *fd;
     Colormap cmap;
 
@@ -414,6 +414,9 @@ saveConfigOkCallback(Widget il, LocalInfo * info, char *file)
     }
     fprintf(fd, "reset\n\n");
 
+    GetDefaultWH(&w, &h);
+    fprintf(fd, "DefaultSize %dx%d\n\n", w, h);
+
     for (i = 0; i < nchildren; i++) {
 	PatternInfo *pi;
 
@@ -1812,6 +1815,7 @@ PatternEdit(Widget w, Pixel *pixels, Pixmap *patterns, void *brushes,
     XtUnmanageChild(info->cpick);
     XtUnmanageChild(grabPattern);
     XtUnmanageChild(selectPattern);
+    XtUnmanageChild(asbrushPattern);
     XtUnmanageChild(info->r_arrow);
     XtUnmanageChild(info->g_arrow);
     XtUnmanageChild(info->view1);
@@ -1840,6 +1844,7 @@ PatternEdit(Widget w, Pixel *pixels, Pixmap *patterns, void *brushes,
     XMapWindow(dpy, XtWindow(info->cpick));
     XMapWindow(dpy, XtWindow(grabPattern));
     XMapWindow(dpy, XtWindow(selectPattern));
+    XMapWindow(dpy, XtWindow(asbrushPattern));
     XMapWindow(dpy, XtWindow(info->view1));
     XMapWindow(dpy, XtWindow(info->view2));
     XMapWindow(dpy, XtWindow(info->icon));
