@@ -115,6 +115,7 @@ press(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 	l->lgc = info->first_gc;
 
     } else if (event->button == Button2 && l->tracking) {
+        l->drawn = True;
 	finish(w, l, True);
 	return;
     }
@@ -175,7 +176,7 @@ motion(Widget w, LocalInfo * l, XMotionEvent * event, OpInfo * info)
     l->endX = event->x;
     l->endY = event->y;
 
-    if ((l->drawn = (l->startX != l->endX && l->startY != l->endY)))
+    if ((l->drawn = (l->startX != l->endX || l->startY != l->endY)))
 	XDrawLine(XtDisplay(w), info->drawable, l->gcx,
 		  l->startX, l->startY, l->endX, l->endY);
 }
