@@ -647,17 +647,28 @@ hmminusCallback(Widget w, XtPointer larg, XtPointer junk)
 
 void createPrintData() 
 {
-    char buf[120];
+    int fd;
     char *tmp;
+    char buf[256];
+
     printinfo = (PrintInfo *) XtMalloc(sizeof(PrintInfo));
     printinfo->tied = True;
     printinfo->orient = False;
     printinfo->output = False;
     printinfo->page = 0;
-    tmpnam(buf);
-    if ((tmp = getenv("TMPDIR")) == NULL) tmp = "/tmp";
-    printinfo->tmpfile = (char *) xmalloc(30+strlen(tmp));
-    sprintf(printinfo->tmpfile, "%s/Xpaint-%s.ps", tmp, rindex(buf, '/')+1);
+
+    if ((tmp = getenv("TMPDIR")) == NULL)
+       tmp = P_tmpdir;
+
+    strncpy(buf, tmp, 220);
+    buf[220] = '\0';
+    strcat(buf, "/XPaint-XXXXXX");
+    fd = mkstemp(buf);
+    if (fd == -1) return;
+    close(fd);
+    unlink(buf);
+    strcat(buf, ".ps");
+    printinfo->tmpfile = XtNewString(buf);
 }
 
 void 
@@ -1218,9 +1229,9 @@ viewExternCallback(Widget w, XtPointer infoArg, XtPointer junk2)
     ExternInfo *info = (ExternInfo *) infoArg;
     Image *image;
     static char suffix[8];
-    char cmd[512], buf[120];
+    char cmd[512], buf[256];
     char *viewer, *tmp, *sep, *file = NULL;
-    int i, del;
+    int i, del, fd;
 
     XtVaGetValues(info->viewer, XtNstring, &viewer, NULL);
     if (!viewer || !*viewer) return;
@@ -1230,7 +1241,10 @@ viewExternCallback(Widget w, XtPointer infoArg, XtPointer junk2)
     XtVaGetValues(info->formatChecks[info->mode], XtNlabel, &tmp, NULL);
     strcpy(suffix, tmp);
     i = 0;
-    while (suffix[i]) suffix[i++] = tolower(suffix[i]);
+    while (suffix[i]) {
+       suffix[i] = tolower(suffix[i]);
+       i++;
+    }
     del = 1;
     sep = rindex(info->cmd, ' ');
     if (sep) {
@@ -1244,11 +1258,18 @@ viewExternCallback(Widget w, XtPointer infoArg, XtPointer junk2)
     }
 
     if (!file) {
-        *buf = '\0';
-        tmpnam(buf);
-        if ((tmp = getenv("TMPDIR")) == NULL) tmp = "/tmp";
-        file = (char *) xmalloc(30+strlen(tmp));
-        sprintf(file, "%s/XPaint-%s.%s", tmp, rindex(buf, '/')+1, suffix);
+       if ((tmp = getenv("TMPDIR")) == NULL)
+          tmp = P_tmpdir;
+        strncpy(buf, tmp, 220);
+        buf[220] = '\0';
+        strcat(buf, "/XPaint-XXXXXX");
+        fd = mkstemp(buf);
+        if (fd == -1) return;
+        close(fd);
+        unlink(buf);
+        strcat(buf, ".");
+	strcat(buf, suffix);
+        file = XtNewString(buf);
     }
     if (del)
         sprintf(cmd, "(%s %s ; rm -f %s) &", info->cmd, file, file);
