@@ -1,18 +1,19 @@
 /* +-------------------------------------------------------------------+ */
-/* | Copyright 1996-2000, Greg Roelofs (newt@pobox.com)                | */
-/* | Last revised:  26 August 2000                                     | */
+/* | Copyright 1996-2003, Greg Roelofs (newt@pobox.com)                | */
+/* | Last revised:  1 June 2003                                        | */
 /* +-------------------------------------------------------------------+ */
 
+/* maintainer can nuke this line if not using CVS: */
 /* $Id: readWritePNG.c,v 1.1 2000/09/02 22:09:06 torsten Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
-#include "png.h"
-#include "image.h"
-#include "rwTable.h"
+#include <png.h>
 
 typedef char *String;
-#include "../messages.h"
+#include <messages.h>
+#include <image.h>
+#include "rwTable.h"
 
 #ifndef TRUE
 #  define TRUE 1
@@ -64,8 +65,8 @@ ReadPNG(char *file)
     png_uint_32   width, height;
     png_colorp    palette;
     int           bit_depth, color_type, interlace_type, num_palette, rowbytes;
-    int           i, hasAlpha=FALSE;
-    int           level, npasses;
+    int           i, hasAlpha;
+    int           level /* , npasses */ ;
     Image         *image = NULL;
 
 
@@ -94,7 +95,7 @@ ReadPNG(char *file)
     if (setjmp(xpaint_jmpbuf_struct.jmpbuf)) {
         RWSetMsg(msgText[FATAL_LIBPNG_ERROR_LONGJMP_CALLED_WHILE_READING]);
         fprintf(stderr, "%s\n",
-		msgText[FATAL_LIBPNG_ERROR_LONGJMP_CALLED_WHILE_READING]);
+          msgText[FATAL_LIBPNG_ERROR_LONGJMP_CALLED_WHILE_READING]);
         fflush(stderr);
         png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
         fclose(fp);
@@ -108,6 +109,8 @@ ReadPNG(char *file)
       &interlace_type, NULL, NULL);
     Trace((stderr, "GRR ReadPNG:  width = %d, height = %d\n", width, height));
 
+    hasAlpha = FALSE;
+
     switch (color_type) {
 
         case PNG_COLOR_TYPE_PALETTE:
@@ -115,7 +118,7 @@ ReadPNG(char *file)
             if (!png_get_PLTE(png_ptr, info_ptr, &palette, &num_palette)) {
                 RWSetMsg(msgText[ERROR_PLTE_CHUNK_NOT_FOUND_IN_PALETTE_IMAGE]);
                 fprintf(stderr, "%s\n",
-		        msgText[ERROR_PLTE_CHUNK_NOT_FOUND_IN_PALETTE_IMAGE]);
+                  msgText[ERROR_PLTE_CHUNK_NOT_FOUND_IN_PALETTE_IMAGE]);
                 fflush(stderr);
                 png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
                 fclose(fp);
@@ -133,7 +136,7 @@ ReadPNG(char *file)
             if (bit_depth == 16) {
                 RWSetMsg(msgText[STRIPPING_48_BIT_RGB_IMAGE_TO_24_BITS]);
                 fprintf(stderr, "%s\n",
-                        msgText[STRIPPING_48_BIT_RGB_IMAGE_TO_24_BITS]);
+                  msgText[STRIPPING_48_BIT_RGB_IMAGE_TO_24_BITS]);
                 fflush(stderr);
                 png_set_strip_16(png_ptr);
             }
@@ -145,7 +148,7 @@ ReadPNG(char *file)
             if (bit_depth == 16) {
                 RWSetMsg(msgText[STRIPPING_16_BIT_GRAYSCALE_IMAGE_TO_8_BITS]);
                 fprintf(stderr, "%s\n",
-                        msgText[STRIPPING_16_BIT_GRAYSCALE_IMAGE_TO_8_BITS]);
+                  msgText[STRIPPING_16_BIT_GRAYSCALE_IMAGE_TO_8_BITS]);
                 fflush(stderr);
                 png_set_strip_16(png_ptr);
                 bit_depth = 8;
@@ -184,7 +187,7 @@ ReadPNG(char *file)
             if (bit_depth == 16) {
                 RWSetMsg(msgText[STRIPPING_64_BIT_RGBA_IMAGE_TO_32_BITS]);
                 fprintf(stderr, "%s\n",
-                        msgText[STRIPPING_64_BIT_RGBA_IMAGE_TO_32_BITS]);
+                  msgText[STRIPPING_64_BIT_RGBA_IMAGE_TO_32_BITS]);
                 fflush(stderr);
                 png_set_strip_16(png_ptr);
             }
@@ -199,7 +202,7 @@ ReadPNG(char *file)
             if (bit_depth == 16) {
                 RWSetMsg(msgText[STRIPPING_32_BIT_GRAY_ALPHA_IMAGE_TO_16_BITS]);
                 fprintf(stderr, "%s\n",
-                        msgText[STRIPPING_32_BIT_GRAY_ALPHA_IMAGE_TO_16_BITS]);
+                  msgText[STRIPPING_32_BIT_GRAY_ALPHA_IMAGE_TO_16_BITS]);
                 fflush(stderr);
                 png_set_strip_16(png_ptr);
             }
@@ -210,7 +213,7 @@ ReadPNG(char *file)
 
         default:
             fprintf(stderr, "ReadPNG:  %s (%d)\n", 
-                    msgText[UNKNOWN_PNG_IMAGE_TYPE], color_type);
+              msgText[UNKNOWN_PNG_IMAGE_TYPE], color_type);
             fflush(stderr);
             RWSetMsg(msgText[UNKNOWN_PNG_IMAGE_TYPE]);
             png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
@@ -222,7 +225,7 @@ ReadPNG(char *file)
         png_set_packing(png_ptr);
 
     if (interlace_type)
-        npasses = png_set_interlace_handling(png_ptr);
+        /* npasses = */ png_set_interlace_handling(png_ptr);
 
     png_read_update_info(png_ptr, info_ptr);
 
@@ -231,19 +234,18 @@ ReadPNG(char *file)
     if (hasAlpha) {
         png_bytep *row_pointers, png_data;
 
-	if ((row_pointers =
-             (png_bytep *)malloc(height * sizeof(png_bytep))) == 0)
-        {
-	    RWSetMsg(msgText[READPNG_CANT_ALLOCATE_MEMORY_FOR_ROW_POINTERS_1]);
+        row_pointers = (png_bytep *)malloc(height * sizeof(png_bytep));
+        if (!row_pointers) {
+            RWSetMsg(msgText[READPNG_CANT_ALLOCATE_MEMORY_FOR_ROW_POINTERS_1]);
             fprintf(stderr, "%s\n",
-                    msgText[READPNG_CANT_ALLOCATE_MEMORY_FOR_ROW_POINTERS_1]);
+              msgText[READPNG_CANT_ALLOCATE_MEMORY_FOR_ROW_POINTERS_1]);
             fflush(stderr);
             ImageDelete(image);
             png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
             fclose(fp);
-	    return NULL;
-	}
-	
+            return NULL;
+        }
+
         /* very inefficient (memory-wise) to allocate entire image again, but
          * no easy way around it:  libpng returns the image and alpha channel
          * interspersed, and interlaced alpha images just make matters worse
@@ -251,9 +253,10 @@ ReadPNG(char *file)
         png_data = (png_bytep)malloc(height*rowbytes);
         if (!png_data) {
             RWSetMsg(
-                msgText[UNABLE_TO_ALLOCATE_TEMPORARY_STORAGE_FOR_ALPHA_IMAGE]);
+              msgText[UNABLE_TO_ALLOCATE_TEMPORARY_STORAGE_FOR_ALPHA_IMAGE]);
             fprintf(stderr, msgText[READPNG_UNABLE_TO_MALLOC_PNG_DATA]);
             fflush(stderr);
+            free(row_pointers);
             ImageDelete(image);
             png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
             fclose(fp);
@@ -284,26 +287,28 @@ ReadPNG(char *file)
             }
         }
         free(png_data);   /* whew */
+        free(row_pointers);
 
     } else {  /* no alpha channel */
         png_bytep *row_pointers;
 
-	if ((row_pointers =
-             (png_bytep *)malloc(height * sizeof(png_bytep))) == 0)
-        {
-	    RWSetMsg(msgText[READPNG_CANT_ALLOCATE_MEMORY_FOR_ROW_POINTERS_2]);
+        row_pointers = (png_bytep *)malloc(height * sizeof(png_bytep));
+        if (!row_pointers) {
+            RWSetMsg(msgText[READPNG_CANT_ALLOCATE_MEMORY_FOR_ROW_POINTERS_2]);
             fprintf(stderr, "%s\n",
-                msgText[READPNG_CANT_ALLOCATE_MEMORY_FOR_ROW_POINTERS_2]);
+              msgText[READPNG_CANT_ALLOCATE_MEMORY_FOR_ROW_POINTERS_2]);
             fflush(stderr);
             ImageDelete(image);
             png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
             fclose(fp);
-	    return NULL;
-	}
+            return NULL;
+        }
+
         for (i = 0;  i < height;  ++i)
             row_pointers[i] = (png_bytep)image->data + i*rowbytes;
 
         png_read_image(png_ptr, row_pointers);
+        free(row_pointers);
 
     } /* end if (hasAlpha) */
 
@@ -323,6 +328,11 @@ ReadPNG(char *file)
 
 
 
+/* GRR 20010720:  Note that image gets discarded immediately after we return
+ *   [unless somebody, presumably us, boosts the refCount to save it?  --see
+ *   stdSaveCommonCallback() in fileName.c], so what we do to it here matters
+ *   only to us. */
+
 static int
 WritePNG(char *file, Image *image, int interlace_type)
 {
@@ -358,7 +368,7 @@ WritePNG(char *file, Image *image, int interlace_type)
     if (setjmp(xpaint_jmpbuf_struct.jmpbuf)) {
         RWSetMsg(msgText[FATAL_LIBPNG_ERROR_LONGJMP_CALLED_WHILE_WRITING]);
         fprintf(stderr, "%s\n",
-            msgText[FATAL_LIBPNG_ERROR_LONGJMP_CALLED_WHILE_WRITING]);
+          msgText[FATAL_LIBPNG_ERROR_LONGJMP_CALLED_WHILE_WRITING]);
         fflush(stderr);
         png_destroy_write_struct(&png_ptr, &info_ptr);
         fclose(fp);
@@ -367,6 +377,9 @@ WritePNG(char *file, Image *image, int interlace_type)
 
     png_init_io(png_ptr, fp);
 
+    color_type = -1;  /* quiet compiler warning (all cases actually covered) */
+    bit_depth = -1;
+
     if (image->isBW) {
         if (image->maskData) {
             color_type = PNG_COLOR_TYPE_GRAY_ALPHA;
@@ -383,11 +396,12 @@ WritePNG(char *file, Image *image, int interlace_type)
         if (image->cmapPacked)
             bit_depth = 8;
         else {
-            Trace((stderr,
-              "GRR WritePNG:  isGrey, cmapSize = %d (before compressing), ",
-              image->cmapSize));
-            compressColormap(image);
-            Trace((stderr, "%d (after)\n", image->cmapSize));
+            Trace((stderr, "GRR WritePNG:  isGrey, cmapSize = %d and scale = "
+              "%d (before compressing),\n", image->cmapSize, image->scale));
+            compressColormap(image);   /* GRR:  destructive?? (seems OK...) */
+            Trace((stderr, "               %s = %d and scale = %d (after "
+              "compressing)\n", image->isGrey? "isGrey, cmapSize" :
+              "NOT isGrey, cmap", image->cmapSize, image->scale));
             if (image->cmapSize > 16)
                 bit_depth = 8;
             else if (image->cmapSize > 4)
@@ -396,8 +410,8 @@ WritePNG(char *file, Image *image, int interlace_type)
                 bit_depth = 2;
             else
                 bit_depth = 1;
-            Trace((stderr, "GRR WritePNG:  isGrey, picked bit_depth = %d\n",
-              bit_depth));
+            Trace((stderr, "GRR WritePNG:  [was] isGrey; picked bit_depth = "
+              "%d\n", bit_depth));
         }
 
     } else if (image->scale == 3) {
@@ -424,41 +438,85 @@ WritePNG(char *file, Image *image, int interlace_type)
         color_type = PNG_COLOR_TYPE_PALETTE;
         if (image->maskData) {
             fprintf(stderr,
-                msgText[WRITEPNG_CANT_USE_ALPHA_MASK_WITH_COLORMAPPED_IMAGE]);
+              msgText[WRITEPNG_CANT_USE_ALPHA_MASK_WITH_COLORMAPPED_IMAGE]);
             fflush(stderr);
         }
-        Trace((stderr, "GRR WritePNG:  palette, bit_depth = "));
+        Trace((stderr, "GRR WritePNG:  palette..."));
         if (!image->cmapPacked)
-            compressColormap(image);
-        if (image->cmapSize > 16)
+            compressColormap(image);   /* apparently not destructive... */
+
+        if (image->cmapSize > 16) {
+            unsigned char *p;
+            int must_remap = FALSE;
+
             bit_depth = 8;
-        else if (image->cmapSize > 4)
+            /* if palette has more than 16 entries, all of which are gray, it's
+             * more efficient to omit palette entirely and write a grayscale
+             * PNG */
+            color_type = PNG_COLOR_TYPE_GRAY;
+            p = image->cmapData;   /* 1D array of RGB triplets */
+            for (i = image->cmapSize;  i > 0;  --i, p += 3) {
+                /*   red != green ||   red != blue */
+                if (p[0] != p[1]  ||  p[0] != p[2]) {
+                    color_type = PNG_COLOR_TYPE_PALETTE;  /* not gray, alas */
+                    break;
+                }
+                /* red = green = blue, but if != index, will have to remap */
+                if (p[0] != i)
+                    must_remap = TRUE;
+            }
+            if (color_type == PNG_COLOR_TYPE_GRAY) {
+                Trace((stderr, "no, grayscale..."));
+                if (must_remap) {
+                    unsigned char *q;
+
+                    /* reset all pixels to [red] palette value indexed by
+                     * current pixel value (now know red == green == blue) */
+                    Trace((stderr, "remapping..."));
+                    p = image->cmapData;  /* 1D array of RGB (ggg) triplets */
+                    q = image->data;      /* 1D array of indices into palette */
+                    for (i = image->width * image->height;  i > 0;  --i, ++q) {
+                        *q = p[3*(*q)];
+                    }
+                }
+            }
+        } else if (image->cmapSize > 4)
             bit_depth = 4;
         else if (image->cmapSize > 2)
             bit_depth = 2;
         else
             bit_depth = 1;
-        palette = (png_colorp)image->cmapData;   /* GRR:  seems to work... */
+
+        if (color_type == PNG_COLOR_TYPE_PALETTE) {
+            palette = (png_colorp)image->cmapData;   /* GRR:  seems to work */
+            num_palette = image->cmapSize;
 #if 0
-0000000000000000000
-        for (i = 0;  i < image->cmapSize;  ++i) {
-            palette[i].red = image->cmapData ...
-            palette[i].green = 
-            palette[i].blue = 
-        }
+            for (i = 0;  i < num_palette;  ++i) {
+                palette[i].red = image->cmapData ...
+                palette[i].green = 
+                palette[i].blue = 
+            }
 #endif
-        num_palette = image->cmapSize;
-        png_set_PLTE(png_ptr, info_ptr, palette, num_palette);
-        Trace((stderr, "%d, num_palette = %d\n", bit_depth, num_palette));
+            png_set_PLTE(png_ptr, info_ptr, palette, num_palette);
+            Trace((stderr, "bit_depth = %d, num_palette = %d\n", bit_depth,
+              num_palette));
+        } else /* (color_type == PNG_COLOR_TYPE_GRAY) */ {
+            Trace((stderr, "bit_depth = %d\n", bit_depth));
+            /* recall that image gets discarded immediately after return, so
+             * this won't hurt anybody else */
+            image->isGrey = TRUE;
+        }
     }
 
-    Trace((stderr, "GRR WritePNG:  image->scale = %d, image->width = %d, image->height = %d\n", image->scale, image->width, image->height));
+    Trace((stderr, "GRR WritePNG:  isGrey = %s, scale = %d, width = %d, height"
+      " = %d\n", image->isGrey? "TRUE":"FALSE", image->scale, image->width,
+      image->height));
 
     png_set_IHDR(png_ptr, info_ptr, image->width, image->height, bit_depth,
       color_type, interlace_type, PNG_COMPRESSION_TYPE_DEFAULT,
       PNG_FILTER_TYPE_DEFAULT);
 
-#if 0	/* if we don't know what the gamma is, don't guess */
+#if 0	/* if we don't know what the display-system gamma is, don't guess */
     if (xpaint_foo->gamma > 0.0)
         png_set_gAMA(png_ptr, info_ptr, xpaint_foo->gamma);
 #endif
@@ -500,26 +558,26 @@ WritePNG(char *file, Image *image, int interlace_type)
         png_destroy_write_struct(&png_ptr, &info_ptr);
         fclose(fp);
         return 1;
+
     } else {
         int rowbytes = image->scale * image->width;
         png_bytep *row_pointers;
 
-	if ((row_pointers =
-	     (png_bytep *)malloc(image->height * sizeof(png_bytep))) == 0)
-        {
-	    RWSetMsg(msgText[WRITEPNG_CANT_ALLOCATE_MEMORY_FOR_ROW_POINTERS]);
+        row_pointers = (png_bytep *)malloc(image->height * sizeof(png_bytep));
+        if (!row_pointers) {
+            RWSetMsg(msgText[WRITEPNG_CANT_ALLOCATE_MEMORY_FOR_ROW_POINTERS]);
             fprintf(stderr, "%s\n",
-                msgText[WRITEPNG_CANT_ALLOCATE_MEMORY_FOR_ROW_POINTERS]);
+              msgText[WRITEPNG_CANT_ALLOCATE_MEMORY_FOR_ROW_POINTERS]);
             fflush(stderr);
             png_destroy_write_struct(&png_ptr, &info_ptr);
             fclose(fp);
-	    return 1;
-	}
+            return 1;
+        }
 
         for (i = 0;  i < image->height;  ++i)
             row_pointers[i] = (png_bytep)image->data + i*rowbytes;
 
-#ifdef XPAINT_PNG_DEBUG
+#ifdef XPAINT_PNG_DEBUG___disabled_for_now
         {
             int wrapcount = (image->scale == 1)? 20 : 24;
             int rownum = (image->height >> 1);
@@ -536,6 +594,7 @@ WritePNG(char *file, Image *image, int interlace_type)
 #endif
 
         png_write_image(png_ptr, row_pointers);
+        free(row_pointers);
     }
 
     png_write_end(png_ptr, NULL);
@@ -568,26 +627,26 @@ WritePNGi(char *file, Image *image)
 static void
 xpaint_png_error_handler (png_structp png_ptr, png_const_charp msg)
 {
-  jmpbuf_wrapper  *jmpbuf_ptr;
+    jmpbuf_wrapper  *jmpbuf_ptr;
 
-  /* this function, aside from the extra step of retrieving the "error
-   * pointer" (below) and the fact that it exists within the application
-   * rather than within libpng, is essentially identical to libpng's
-   * default error handler.  The second point is critical:  since both
-   * setjmp() and longjmp() are called from the same code, they are
-   * guaranteed to have compatible notions of how big a jmp_buf is,
-   * regardless of whether _BSD_SOURCE or anything else has (or has not)
-   * been defined. */
+    /* this function, aside from the extra step of retrieving the "error
+     * pointer" (below) and the fact that it exists within the application
+     * rather than within libpng, is essentially identical to libpng's
+     * default error handler.  The second point is critical:  since both
+     * setjmp() and longjmp() are called from the same code, they are
+     * guaranteed to have compatible notions of how big a jmp_buf is,
+     * regardless of whether _BSD_SOURCE or anything else has (or has not)
+     * been defined. */
 
-  fprintf(stderr, msgText[XPAINT_FATAL_LIBPNG_ERROR], msg);
-  fflush(stderr);
-
-  jmpbuf_ptr = png_get_error_ptr(png_ptr);
-  if (jmpbuf_ptr == NULL) {         /* we are completely hosed now */
-    fprintf(stderr, msgText[XPAINT_EXTREMELY_FATAL_LIBPNG_ERROR]);
+    fprintf(stderr, msgText[XPAINT_FATAL_LIBPNG_ERROR], msg);
     fflush(stderr);
-    exit(99);
-  }
 
-  longjmp(jmpbuf_ptr->jmpbuf, 1);
+    jmpbuf_ptr = png_get_error_ptr(png_ptr);
+    if (jmpbuf_ptr == NULL) {         /* we are completely hosed now */
+        fprintf(stderr, msgText[XPAINT_EXTREMELY_FATAL_LIBPNG_ERROR]);
+        fflush(stderr);
+        exit(99);
+    }
+
+    longjmp(jmpbuf_ptr->jmpbuf, 1);
 }
