@@ -64,6 +64,7 @@ extern Image *ReadXPM(char *);
 extern Image *ReadTIFF(char *);
 extern Image *ReadJPEG(char *);
 extern Image *ReadICO(char *);
+extern Image *ReadScriptC(char *);
 extern Image *ReadPS(char *);
 extern int WriteXBM(char *, Image *);
 extern int WriteXWD(char *, Image *);
@@ -88,6 +89,7 @@ extern int TestSGI(char *);
 extern int TestTIFF(char *);
 extern int TestJPEG(char *);
 extern int TestICO(char *);
+extern int TestScriptC(char *);
 extern int TestPS(char *);
 extern int TestPDF(char *);
 
@@ -119,6 +121,7 @@ static ImageTypes RWtable[] =
     {"XBM_Format",       ReadXBM,   WriteXBM,  TestXBM },
     {"XPM_Format",       ReadXPM,   WriteXPM,  TestXPM },
     {"XWD_Format",       ReadXWD,   WriteXWD,  TestXWD },
+    {"CSC_Format",       ReadScriptC,   NULL,  TestScriptC }
 };
 
 #define	NUMBER	(sizeof(RWtable) / sizeof(RWtable[0]))
@@ -291,3 +294,9 @@ void RWSetMsg(char *fmt, ...)
     vsprintf(RWtableMsg, fmt, ap);
 }
 #endif
+
+Image *
+ImageFromFile(char *file)
+{
+    return readMagic(file);
+}
