@@ -188,6 +188,11 @@ keyPress(Widget w, LocalInfo * l, XKeyEvent * event, OpInfo * info)
     case XK_Right:
 	dx = 1;
 	break;
+    case XK_Alt_L:
+    case XK_Alt_R:
+        PwRegionTear(w);
+	RegionTransparency((PaintWidget) w, 0);
+	return;
     default:
 	break;
     }
@@ -210,6 +215,19 @@ keyPress(Widget w, LocalInfo * l, XKeyEvent * event, OpInfo * info)
     }
 }
 
+static void 
+keyRelease(Widget w, LocalInfo * l, XKeyEvent * event, OpInfo * info)
+{
+    KeySym keysym;
+    int len, i, dx, dy;
+    char buf[21];
+
+    len = XLookupString(event, buf, sizeof(buf) - 1, &keysym, NULL);
+
+    if (keysym==XK_Alt_L || keysym==XK_Alt_R)
+       RegionTransparency((PaintWidget) w, 1);
+}
+
 static void 
 finishPolyBand(Widget w, LocalInfo * l, OpInfo * info)
 {
@@ -743,6 +761,8 @@ SelectBoxAdd(Widget w)
 		      (OpEventProc) release, l);
     OpAddEventHandler(w, opWindow, KeyPressMask, FALSE,
 		      (OpEventProc) keyPress, l);
+    OpAddEventHandler(w, opWindow, KeyReleaseMask, FALSE,
+		      (OpEventProc) keyRelease, l);
     SetCrossHairCursor(w);
 
     return l;
@@ -761,6 +781,8 @@ SelectBoxRemove(Widget w, void *p)
 			 (OpEventProc) release, l);
     OpRemoveEventHandler(w, opWindow, KeyPressMask, FALSE,
 			 (OpEventProc) keyPress, l);
+    OpRemoveEventHandler(w, opWindow, KeyReleaseMask, FALSE,
+			 (OpEventProc) keyRelease, l);
     PwRegionFinish(w, False);
 
     XtFree((XtPointer) l->window_p);
@@ -791,6 +813,8 @@ LassoAdd(Widget w)
 		      (OpEventProc) release, l);
     OpAddEventHandler(w, opWindow, KeyPressMask, FALSE,
 		      (OpEventProc) keyPress, l);
+    OpAddEventHandler(w, opWindow, KeyReleaseMask, FALSE,
+		      (OpEventProc) keyRelease, l);
 
     SetPencilCursor(w);
 
@@ -810,6 +834,8 @@ LassoRemove(Widget w, void *p)
 			 (OpEventProc) release, l);
     OpRemoveEventHandler(w, opWindow, KeyPressMask, FALSE,
 			 (OpEventProc) keyPress, l);
+    OpRemoveEventHandler(w, opWindow, KeyReleaseMask, FALSE,
+			 (OpEventProc) keyRelease, l);
     PwRegionFinish(w, False);
 
     XtFree((XtPointer) l->window_p);
@@ -844,7 +870,8 @@ SelectPolyAdd(Widget w)
 		      (OpEventProc) releasePolyBand, l);
     OpAddEventHandler(w, opWindow, KeyPressMask, FALSE,
 		      (OpEventProc) keyPress, l);
-
+    OpAddEventHandler(w, opWindow, KeyReleaseMask, FALSE,
+		      (OpEventProc) keyRelease, l);
     SetPencilCursor(w);
 
     return l;
@@ -863,6 +890,8 @@ SelectPolyRemove(Widget w, void *p)
 			 (OpEventProc) releasePolyBand, l);
     OpRemoveEventHandler(w, opWindow, KeyPressMask, FALSE,
 			 (OpEventProc) keyPress, l);
+    OpRemoveEventHandler(w, opWindow, KeyReleaseMask, FALSE,
+			 (OpEventProc) keyRelease, l);
     PwRegionFinish(w, False);
 
     XtFree((XtPointer) l->window_p);
