@@ -98,7 +98,11 @@ okSizeCallback(Widget w, XtPointer argArg, XtPointer infoArg)
     } else if (arg->h <= 0) {
 	Notice(w, msgText[INVALID_HEIGHT]);
     } else if (arg->paint == None) {
-	arg->func(arg->widget, arg->w, arg->h, arg->z);
+        if (arg->func)
+	    arg->func(arg->widget, arg->w, arg->h, arg->z);
+	else {
+            SetDefaultWHZ(arg->w, arg->h, arg->z);
+	}
     } else if (arg->w != width || arg->h != height) {
 	AlertBox(GetShell(arg->paint),
 		 msgText[RESIZING_WARNING_CANNOT_BE_UNDONE],
