@@ -585,7 +585,7 @@ static void ComputeLayout(widget, query, destroy_scrollbars)
 	     */
 #define CheckHoriz()							  \
 	    if ( w->viewport.allowhoriz &&				  \
-		 (int)preferred.width > clip_width) { 			  \
+		 (int)preferred.width > clip_width+4) {			  \
 	        if (!needshoriz) {					  \
 		    Widget bar;					          \
 		    needshoriz = True;				          \
@@ -599,7 +599,7 @@ static void ComputeLayout(widget, query, destroy_scrollbars)
 	    }
 /*enddef*/
 	    CheckHoriz();
-	    if (w->viewport.allowvert && (int)preferred.height > clip_height) {
+	    if (w->viewport.allowvert && (int)preferred.height > clip_height+4) {
 	        if (!needsvert) {
 		    Widget bar;
 		    needsvert = True;
